/*
 * PGrid.java
 *
 * Abstract base class for Crypt grid structure.
 */

package sim;

import java.util.ArrayList;

abstract public class PGrid {  
    protected Crypt _crypts[];
    protected CryptFactory _factory;
    abstract public ArrayList<Integer> getNeighbors(int n);
    abstract public double getMaxX();         // width of 2d plane
    abstract public double getMaxY();         // height of 2d plane
    abstract public double getX(int n);       // returns the x-coordinate of crypt n on a plane
    abstract public double getY(int n);       // returns the y-coordinate of crypt n on a plane
    abstract public Crypt getCryptAtCenter(); // returns the crypt at the center of the Grid

    // returns the # of crypts
    public int getSize() { return _crypts.length; }
    
    // returns a single crypt
    public Crypt getCrypt(int x) {
	if (x < getSize())
	    return _crypts[x];
	else
	    return null;
    }

    // Checks the state of each crypt and returns the counts of each state in the array, where
    //	 counters[0] == # Dead, counters[1] == #AliveWT, counters[2] = #Mutant
    public int[] CountCryptStates()
    {
	int[] counters = new int[3];  // counters[0] == # Dead, counters[1] == #AliveWT, counters[2] = #Mutant
		
	for( int i = 0; i < _crypts.length; i++ ) {
	    ++counters[_crypts[i].GetCryptState().ordinal()];
	}
	return counters;		
    }
	
    public int CountCryptsInState(Crypt.eCryptState eState)
    {
	int numCrypts = 0;
	for( int i = 0; i < _crypts.length; i++ ) {
	    if( _crypts[i].GetCryptState() == eState )
		++numCrypts;
	}
	return numCrypts;
    }
    
    // returns crypts in a circular region with radius r, centered at (x,y)
    public ArrayList<Crypt> getCryptsInCircle(double x, double y, double r) {
	ArrayList<Crypt> list = new ArrayList<Crypt>();		
	for(int i=0; i<_crypts.length; i++ )
	    if (Math.hypot(getX(i)-x, getY(i)-y)<=r)
		list.add(_crypts[i]);
	return list;
    }
}
