/*
 * GUITissueDriver
 * Interactive (GUI) version of driver for tissue simulation.
 */

package ui;

import java.io.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import java.text.DateFormat;
import java.util.Date;
import javax.imageio.*;

import sim.*;

@SuppressWarnings("serial")
class ParamDialog extends JDialog
                  implements ActionListener, ItemListener {
  private JFormattedTextField _mutation;
  private JFormattedTextField _dimx;
  private JFormattedTextField _dimy;
  private JFormattedTextField _apoptosis;
  private JFormattedTextField _woundingProb;
  private JFormattedTextField _woundingTime;
  private JFormattedTextField _survival;
  private JFormattedTextField _resistance;
  private JFormattedTextField _reproductive;
  private JFormattedTextField _simTime;
  private JFormattedTextField _random;
  private JTextField _label;

  public ParamDialog(Frame owner, String title) {
    super(owner, title, true);

    setTitle("Initialize tissue simulation");
    setSize(700,600);
    Container contentPane = getContentPane();
    contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));   
    NumberFormat doubleFormat = NumberFormat.getNumberInstance();
    doubleFormat.setMinimumFractionDigits(8);
    NumberFormat intFormat = NumberFormat.getIntegerInstance();

    JPanel LabelPanel = new JPanel(new GridLayout(1,3));
    LabelPanel.add(new JLabel("label:"));
    _label = new JTextField("temp");
    LabelPanel.add(_label);
    LabelPanel.add(new JLabel("label for result output"));
    contentPane.add(LabelPanel);
    
    JPanel RandomPanel = new JPanel(new GridLayout(1,3));
    RandomPanel.add(new JLabel("random number seed:"));
    _random = new JFormattedTextField(intFormat);
    _random.setValue(new Integer(0));
    _random.setColumns(8);
    RandomPanel.add(_random);
    RandomPanel.add(new JLabel("(0 to use clock)"));
    contentPane.add(RandomPanel);

    JPanel DimPanel = new JPanel(new GridLayout(1,3));
    DimPanel.add(new JLabel("dimensions:"));
    _dimx = new JFormattedTextField(intFormat);
    _dimx.setValue(new Integer(80));
    _dimx.setColumns(8);
    DimPanel.add(_dimx);
    DimPanel.add(new JLabel(" * "));
    _dimy = new JFormattedTextField(intFormat);
    _dimy.setValue(new Integer(100));
    _dimy.setColumns(8);
    DimPanel.add(_dimy);
    DimPanel.add(new JLabel("number of cells"));
    contentPane.add(DimPanel);

    JPanel MutationPanel = new JPanel(new GridLayout(1,3));
    MutationPanel.add(new JLabel("mutation rate:"));
    _mutation = new JFormattedTextField(doubleFormat);
    _mutation.setValue(new Double(0.0000001));
    _mutation.setColumns(8);
    MutationPanel.add(_mutation);
    MutationPanel.add(new JLabel("mutation rate per day (e.g., 1.0E-8 to 1.0e-4)"));
    contentPane.add(MutationPanel);

    JPanel DeathPanel = new JPanel(new GridLayout(1,3));
    DeathPanel.add(new JLabel("apoptosis rate:"));
    _apoptosis = new JFormattedTextField(doubleFormat);
    _apoptosis.setValue(new Double(0.0005));
    _apoptosis.setColumns(8);
    DeathPanel.add(_apoptosis);
    DeathPanel.add(new JLabel("apoptosis rate per day (e.g., 10E-7)"));
    contentPane.add(DeathPanel);

    JPanel WoundingProbPanel = new JPanel(new GridLayout(1,3));
    WoundingProbPanel.add(new JLabel("wounding impact:"));
    _woundingProb = new JFormattedTextField(doubleFormat);
    _woundingProb.setValue(new Double(0.01));
    _woundingProb.setColumns(8);
    WoundingProbPanel.add(_woundingProb);
    WoundingProbPanel.add(new JLabel("probability of death"));
    contentPane.add(WoundingProbPanel);

    JPanel WoundingTimePanel = new JPanel(new GridLayout(1,3));
    WoundingTimePanel.add(new JLabel("wounding interval:"));
    _woundingTime = new JFormattedTextField(intFormat);
    _woundingTime.setValue(new Integer(0));
    _woundingTime.setColumns(8);
    WoundingTimePanel.add(_woundingTime);
    WoundingTimePanel.add(new JLabel("days between wounding"));
    contentPane.add(WoundingTimePanel);

    JPanel SurvivalPanel = new JPanel(new GridLayout(1,3));
    SurvivalPanel.add(new JLabel("survival advantage:"));
    _survival = new JFormattedTextField(doubleFormat);
    _survival.setValue(new Double(1.0));
    _survival.setColumns(8);
    SurvivalPanel.add(_survival);
    SurvivalPanel.add(new JLabel("mutant survival advantage (0..1)"));
    contentPane.add(SurvivalPanel);

    JPanel ResistancePanel = new JPanel(new GridLayout(1,3));
    ResistancePanel.add(new JLabel("wounding resistance:"));
    _resistance = new JFormattedTextField(doubleFormat);
    _resistance.setValue(new Double(0.25));
    _resistance.setColumns(8);
    ResistancePanel.add(_resistance);
    ResistancePanel.add(new JLabel("mutant wounding resistance (0..1)"));
    contentPane.add(ResistancePanel);

    JPanel ReproductivePanel = new JPanel(new GridLayout(1,3));
    ReproductivePanel.add(new JLabel("reproductive advantage:"));
    _reproductive = new JFormattedTextField(doubleFormat);
    _reproductive.setValue(new Double(0.25));
    _reproductive.setColumns(8);
    ReproductivePanel.add(_reproductive);
    ReproductivePanel.add(new JLabel("mutant reproductive advantage (0..1)"));
    contentPane.add(ReproductivePanel);

    JPanel SimTimePanel = new JPanel(new GridLayout(1,3));
    SimTimePanel.add(new JLabel("simulation time:"));
    _simTime = new JFormattedTextField(intFormat);
    _simTime.setValue(new Integer(50));
    _simTime.setColumns(8);
    SimTimePanel.add(_simTime);
    SimTimePanel.add(new JLabel("years of simulation time"));
    contentPane.add(SimTimePanel);
    
    Panel buttonPanel = new Panel(new FlowLayout());
    JButton buttonOK = new JButton("ok");
    buttonOK.setActionCommand("ok");
    buttonOK.addActionListener(this);
    buttonPanel.add(buttonOK);
    contentPane.add(buttonPanel);
  }

  public void actionPerformed(ActionEvent e) {
    System.out.println(e.getActionCommand());
    if (e.getActionCommand().equals("ok"))
      setVisible(false);
  }

  public void itemStateChanged(ItemEvent e) {
    if (e.getStateChange() == ItemEvent.SELECTED) {
      JComboBox cb = (JComboBox)e.getSource();
      String szName = (String)cb.getSelectedItem();
    }
  }

  public double getMutationRate() {
    return ((Number)_mutation.getValue()).doubleValue();
  }

  public double getDeathRate() {
    return ((Number)_apoptosis.getValue()).doubleValue();
  }

  public double getWoundingProb() {
    return ((Number)_woundingProb.getValue()).doubleValue();
  }

  public int getWoundingTime() {
    return ((Number)_woundingTime.getValue()).intValue();
  }

  public double getSurvival() {
    return ((Number)_survival.getValue()).doubleValue();
  }

  public double getResistance() {
    return ((Number)_resistance.getValue()).doubleValue();
  }

  public double getReproductive() {
    return ((Number)_reproductive.getValue()).doubleValue();
  }

  public int getSimTime() {
    return ((Number)_simTime.getValue()).intValue();
  }

  public int getRandomSeed() {
    int i = ((Number)_random.getValue()).intValue();
    if (i<=0)
      i=-1;
    return i;
  }

  public int getGridWidth() {
    return ((Number)_dimx.getValue()).intValue();
  }

  public int getGridHeight() {
    return ((Number)_dimy.getValue()).intValue();
  }

  public String getLabel() {
    return _label.getText();
  }
};

@SuppressWarnings("serial")
public class GUITissueDriver extends JFrame
  implements ActionListener, ISimulationProgressCallback {
  private Scheduler _scheduler;
  private Date _date;

  // UI items
  private ParamDialog _paramDialog;
  private PGridView _gridView;
  private int     _runstate;
  private JLabel  _labelClock;
  private JButton _buttonRun;
  private JButton _buttonStep;
  private static final int 
    RUN = 1,  // run at full speed
    STOP = 2, // stop and wait
    STEP = 3; // clock one cycle and wait

  public GUITissueDriver() {
    System.out.println("# Tissue simulation");
    System.out.println("# This code comes with ABSOLUTELY NO WARRANTY");
    System.out.println("# This is free software, and you are welcome to redistribute it under certain conditions");
    System.out.println("# It has been licensed under the GNU General Public License");
    System.out.println("# see http://www.gnu.org/licenses/licenses.html#GPL for details");
    System.out.println();
    _date = new Date();
    DateFormat fmt = DateFormat.getInstance();
    System.out.println("# Run on " + fmt.format(_date));

    _paramDialog = new ParamDialog(this, "Simulation parameters");
    _paramDialog.setVisible(true);
		
    TissueEnvironmentSettings params = new TissueEnvironmentSettings();

    params.set_meanCryptDivisionTime(2.0);
    params.set_mutantReproductiveAdvantageFactor(_paramDialog.getReproductive());
    params.set_mutantSurvivalAdvantageFactor(_paramDialog.getSurvival());
    params.set_mutantWoundResistanceFactor(_paramDialog.getResistance());
    params.set_mutationRate(_paramDialog.getMutationRate());
    params.set_wtBackgroundDeathRate(_paramDialog.getDeathRate());
    params.set_wtWoundImpactProbability(_paramDialog.getWoundingProb());
    params.set_woundIntervalTime(_paramDialog.getWoundingTime());
    params.set_maxSimulationYears(_paramDialog.getSimTime());
    params.set_label(_paramDialog.getLabel());
    params.set_woundRadius(40.0);

    try
      {
	  PHexGrid grid = new PHexGrid(_paramDialog.getGridWidth(), _paramDialog.getGridHeight(), CryptFactory.getFactory(), params);
	  //	  PSquareGrid grid = new PSquareGrid(_paramDialog.getGridWidth(), _paramDialog.getGridHeight(), CryptFactory.getFactory(), params, false, false);
	  _scheduler = new Scheduler( grid, _paramDialog.getRandomSeed(), params );
	  _scheduler.PostEvent(new SimulationEvent(null, 0, SimulationEvent.EventType.callback));
      }
    catch( Exception ex )
      {
        System.out.println( "Exception caught in GUITissueDriver:GuiTissueDriver(): " + ex.getMessage());
        ex.printStackTrace(System.out);
      }

    initGUI();
    Scheduler.SimulationTerminationReason eTermReason = _scheduler.RunSimulation(this);
  }
	
  private void initGUI() {
    setTitle("Clone proliferation model");
    setSize(1050,750);
    Container contentPane = getContentPane();
    contentPane.setLayout(new BorderLayout());
    _gridView = new PGridView(_scheduler.GetGrid());
    contentPane.add(_gridView, BorderLayout.CENTER);
    Panel buttonPanel = new Panel();
    buttonPanel.setLayout(new GridLayout(6,1));
    Panel clockPanel = new Panel();
    clockPanel.setLayout(new GridLayout(1,2));
    clockPanel.add(new JLabel("years:days="));
    _labelClock = new JLabel("-");
    clockPanel.add(_labelClock);
    buttonPanel.add(clockPanel);
		
    _buttonRun = new JButton("Go");
    _buttonRun.setActionCommand("run");
    _buttonRun.setBackground(Color.green);
    _buttonRun.addActionListener(this);
    buttonPanel.add(_buttonRun);
		
    _buttonStep = new JButton("Step");
    _buttonStep.setActionCommand("step");
    _buttonStep.setBackground(Color.yellow);
    _buttonStep.addActionListener(this);
    buttonPanel.add(_buttonStep);

    JButton buttonSave = new JButton("Save as PNG");
    buttonSave.setBackground(Color.blue);
    buttonSave.setActionCommand("save");
    buttonSave.addActionListener(this);
    buttonPanel.add(buttonSave);

    JButton buttonClose = new JButton("Quit");
    buttonClose.setBackground(Color.white);
    //buttonClose.setMnemonic(KeyEvent.VK_ESCAPE);
    buttonClose.setActionCommand("close");
    buttonClose.addActionListener(this);
    buttonPanel.add(buttonClose);
		
    contentPane.add(buttonPanel, BorderLayout.EAST);
    setVisible(true);
    _runstate = STOP;
  }
	
  public void actionPerformed(ActionEvent e) {
    if (e.getActionCommand().equals("run")) {
      if (_runstate==RUN) {
        _runstate = STOP;
        _buttonRun.setBackground(Color.green);
        _buttonRun.setText("go");
      } else {
        _runstate = RUN;
        _buttonRun.setBackground(Color.red);
        _buttonRun.setText("stop");
      }
    } else if (e.getActionCommand().equals("step")) {
      _runstate = STEP;
      _buttonRun.setBackground(Color.green);
      _buttonRun.setText("go");
    } else if (e.getActionCommand().equals("save")) {
      SaveImage();
    } else if (e.getActionCommand().equals("close")) {
      System.exit(0);
    }
  }

  private void SaveImage()
  {
    try
      {
        // Prompt user for path+filename into which we'll store the image
        JFileChooser fileChooser = new JFileChooser();
        if (fileChooser.showSaveDialog(this) != JFileChooser.APPROVE_OPTION)
          return;
        File file = fileChooser.getSelectedFile();
        Container contentPane = getContentPane();
        int width = contentPane.getWidth();
        int height = contentPane.getHeight();

        BufferedImage img = new BufferedImage( width, height, BufferedImage.TYPE_INT_RGB ); 
        Graphics2D g = img.createGraphics();
        _gridView.paint( g );
        g.dispose();
			
        ImageIO.write(img, "png", file);
      }
    catch( Exception ex )
      {
      }
  }

  /*
   * Interface ISimulationCallback
   */ 
  public int get_progressInterval() {
      return Integer.MAX_VALUE; // we never want callbacks based on number of events
  }

  public void Execute(Object o)
  {
    final NumberFormat intFormat = NumberFormat.getIntegerInstance();
    final DecimalFormat doubleFormat = new DecimalFormat("0.00");

    if (_runstate==STEP)
      _runstate=STOP;
    try
      {
        _gridView.update();
        double day = _scheduler.GetCurrentTime();
        _labelClock.setText(intFormat.format(day/365) + ":" + 
                            doubleFormat.format(day%365) + " ");
        while (_runstate==STOP) {
          Thread.sleep(100);
        }

	// Run simulation for another 365 days
	_scheduler.PostEvent(new SimulationEvent(null, _scheduler.GetCurrentTime()+365, SimulationEvent.EventType.callback));
      }
    catch( Exception ex ) {
      System.out.println( "Exception caught in GUITissueDriver.ISimulationCallback.Execute(): " + ex.getMessage());
      ex.printStackTrace(System.out);			
    }
  }

  /*
   * main - run the driver
   */
  public static void main(String[] args) {
    GUITissueDriver d = new GUITissueDriver();
  }
}
